@@ -2,10 +2,11 @@
 #
 #  script:	prm_list_types.pl
 #  author:	Pieter J. in 't Veld
-#  date:	July 25, 2017, February 27, 2018, April 2, May 26, June 4, 2021
+#  date:	July 25, 2017, February 27, 2018, April 2, May 26, June 4,
+#  		2021, April 6, 2023, February 19, 2024.
 #  purpose:	List the types occuring in an EMC setup script
 #
-#  Copyright (c) 2004-2022 Pieter J. in 't Veld
+#  Copyright (c) 2004-2025 Pieter J. in 't Veld
 #  Distributed under GNU Public License as stated in LICENSE file in EMC root
 #  directory
 #
@@ -15,16 +16,20 @@
 #    20210402	- Introduced excluded group entries
 #    20210526	- Added '&' line extension interpretation
 #    20210604	- Added -debug and -list
+#    20230406	- New version: 2.0
+#    		- Added ITEM INCLUDE interpretation
+#    20240219	- Added fexpand to fopen
 #
 
-$Version = "1.3";
-$Year = "2021";
+$Version = "2.0";
+$Year = "2024";
 $Copyright = "2004-$Year";
-$Date = "June 4, $Year";
+$Date = "February 19, $Year";
 $Author = "Pieter J. in 't Veld";
 $Target = 0;
 $Debug = 0;
 $Quiet = 0;
+$Path = ["."];
 
 # general functions
 
@@ -41,13 +46,13 @@ sub info {
 
 
 sub fopen {
-  my $name = shift(@_);
+  my $name = fexpand(shift(@_));
   my $mode = shift(@_);
   my $file;
   my $result;
 
   error("illegal mode") if (!($mode eq "r" || $mode eq "w"));
-  info("opening \"$name\" for %s", ($mode eq "r" ? "read" : "writ")."ing\n");
+  info("%s \"%s\"\n", ($mode eq "r" ? "read" : "writ")."ing", scrub_dir($name));
   open($file, ($mode eq "r" ? "<" : ">").$name);
   error("cannot open file \"$name\"") if (!scalar($file));
   return $file;
@@ -143,9 +148,165 @@ sub help {
 }
 
 
+# i/o routines
+
+sub scrub_dir {
+  my $result;
+  my @arg;
+
+  if ($^O eq "MSWin32") {
+    my $a = @_[0];
+
+    $a =~ s/\//\\/g;
+    foreach (split("\\\\", $a)) {
+      push(@arg, $_) if ($_ ne "");
+    }
+    $result = join("/", @arg);
+  } else {
+    $result = substr(@_[0], 0, 1) eq "/" ? "/" : "";
+    foreach (split("/", @_[0])) {
+      push(@arg, $_) if ($_ ne "");
+    }
+    $result .= join("/", @arg);
+    $result =~ s/^$ENV{HOME}/~/;
+  }
+  return $result;
+}
+
+
+sub fexpand {
+  return @_[0] if (substr(@_[0],0,1) ne "~");
+  return $ENV{HOME}.substr(@_[0],1) if (substr(@_[0],1,1) eq "/");
+  return $ENV{HOME}."/../".substr(@_[0],1);
+}
+
+
+sub fexist {
+  my $name = fexpand(shift(@_));
+
+  return 1 if (-f $name);
+  foreach (@_) { return 1 if (-f $name.$_); }
+  return 0;
+}
+
+
+sub flocate {
+  my $name = shift(@_);
+  my @ext = ("", @{shift(@_)});
+
+  foreach ("", @_) {
+    my $root = ($_ eq "" ? "" : $_."/").$name;
+    foreach (@ext) {
+      my $file = $root.$_;
+      return $file if (-f fexpand($file));
+    }
+  }
+  return "";
+}
+
+
+sub trim {
+  my $s = shift(@_);
+  $s =~ s/^\s+|\s+$//g;
+  return $s;
+}
+
+
+sub split_data {
+  my $s = trim(@_[0]);
+  my @arg = split(",", $s);
+  my @result = ();
+  my @i = map({index(@arg[0],$_)} (":", "="));
+
+  if (@i[0]>=0 && @i[0]<@i[1]) {
+    @arg = split(" ", $s);
+    push(@result, @arg[0]);
+    $s = substr($s,length(@arg[0]));
+  }
+  foreach (split("\t", $s)) {
+    foreach (split(",", $_)) { 
+      $_ =~ s/^\s+|\s+$//g;
+      foreach ($_ =~ /(".+"|\S+)/g) {
+	push(@result, $_);
+      }
+    }
+  }
+  if (substr(@result[0],0,1) ne "\"") {
+    @arg = split(" ", @result[0]);
+    if (scalar(@arg)>1) {
+      shift(@result);
+      unshift(@result, @arg);
+    }
+  }
+  push (@result, ",") if (substr(@_[0],-1,1) eq ",");
+  @arg = (); 
+  foreach (@result) {
+    last if (substr($_,0,1) eq "#");
+    push(@arg, $_) if ($_ ne "");
+  }
+  return @arg;
+}
+
+
+sub get_data {
+  my $name = shift(@_);
+  my $stream = fopen($name, "r");
+  my $data = scalar(@_) ? shift(@_) : [];
+  my $text;
+
+  foreach (<$stream>) {
+    ++$line; chomp;
+    $_ = trim($_);
+    next if (substr($_,0,1) eq "#");
+
+    if (substr($_,-1,1) eq "&") {
+      $_ = trim(substr($_,0,-1));
+      $text = length($text) ? "$text $_" : $_;
+      next;
+    }
+
+    my @arg = split_data(length($text) ? "$text $_" : $_);
+    push(@{$data}, [@arg]) if (scalar(@arg));
+    $text = "";
+    if (!$read) {
+      next if (@arg[2] eq "comment=true" || @arg[2] eq "comment=1");
+      if (join(" ", @arg[0,1]) eq "ITEM OPTIONS") {
+	$read = 1;
+      } elsif (join(" ", @arg[0,1]) eq "ITEM INCLUDE") {
+	@arg[2] =~ s/^\"|\"$//g;
+	my $file = flocate(@arg[2], [".dat", ".inc"], @{$Path});
+	$data = get_data($file, $data) if ($file ne "");
+	next;
+      }
+      next;
+    }
+    next if (!scalar(@arg));
+    if (@arg[0] eq "ITEM") {
+      if (@arg[1] eq "END") { $read = 0; next; }
+      print("Error: unexpected ITEM in $line of '$name'.\n\n");
+      exit(-1);
+    }
+    if ($read) {
+      if (@arg[0] eq "location") {
+	shift(@arg);
+	foreach (@arg) {
+	  my @a = split("=");
+	  next if (@a[0] ne "include");
+	  @a[1] =~ s/^\"|\"$//g;
+	  push(@{$Path}, @a[1]);
+	}
+      }
+      next;
+    }
+  }
+  close($stream);
+  return $data;
+}
+
+
 sub list_types {
   my %exclude = (alternate => 1, block => 2, random => 3, "&" => 4, "" => 5);
-  my $stream;
+  my $data = get_data($Source);
   my %replicas;
   my %types;
   my $read;
@@ -156,19 +317,8 @@ sub list_types {
   my $n;
   my @a;
 
-  $stream = fopen($Source, "r");
-  foreach (<$stream>) {
-    ++$line; chop;
-    $_ =~ s/^\s+|\s+$//g;
-    next if (substr($_,0,1) eq "#");
-
-    if ($skip) {
-      $skip = substr($_,-1,1) eq "&" ? 1 : 0;
-      next;
-    }
-    $skip = 1 if (substr($_,-1,1) eq "&");
-    
-    my @arg = split(" ");
+  foreach (@{$data}) {
+    my @arg = @{$_};
     if (!$read) {
       next if (@arg[2] eq "comment=true" || @arg[2] eq "comment=1");
       $read = 1 if (join(" ", @arg[0,1]) eq "ITEM GROUPS");
